#include <ros/ros.h>
#include <std_msgs/String.h>
#include <geometry_msgs/Twist.h>

void callback(const std_msgs::String& msg, geometry_msgs::Twist& cmd) {
  if (msg.data == "avance") {
    cmd.linear.x = 1.0;
    cmd.angular.z = 0.0;
  } else if (msg.data == "arret") {
    cmd = geometry_msgs::Twist();
  } else if (msg.data == "gauche") {
    cmd.angular.z = 1.0;
    cmd.linear.x = 0.0;
  } else if (msg.data == "droite") {
    cmd.angular.z = -1.0;
    cmd.linear.x = 0.0;
  }
}

int main(int argc, char** argv) {
  ros::init(argc, argv, "controller");

  ros::NodeHandle n;
  ros::Publisher pub = n.advertise<geometry_msgs::Twist>("/cmd_vel", 0);

  geometry_msgs::Twist cmd;
  ros::Rate rate(10);

  boost::function<void(const std_msgs::String&)> cb = boost::bind(callback, _1, boost::ref(cmd));
  ros::Subscriber sub = n.subscribe<std_msgs::String&>("/order", 0, cb);

  while(ros::ok()) {
    pub.publish(cmd);
    ros::spinOnce();
    rate.sleep();
  }
  return 0;
}
