#pragma once

#include <stdexcept>
#include <queue>
#include <opencv2/opencv.hpp>
#include <iostream>
#include <map>

#include "file.hpp"
#include "math.hpp"

double distance(const math::csignal& v1, const math::csignal& v2, int n){
  if (v1.size() != v2.size()) {
    throw std::runtime_error("les deux vecteurs doivent être de même longueur");
  }
  double d = 0;
  auto v1_it = v1.begin();
  auto v2_it = v2.begin();

  while (v1_it != v1.end()) {
    double dist = std::abs(*(v1_it++) - *(v2_it++));
    d += std::pow(dist, n);
  }
  return std::pow(d, 1/float(n));
};

struct pair_comp {
  bool operator()(std::pair<double, std::string> a, std::pair<double, std::string> b) {
    if (a.first == b.first) {
      return false;
    }
    if (a.first > b.first) {
      return true;
    }
    return false;
  };
};

std::string predict(const math::csignal& sample, const math::dataset& references, int k, int n) {
  std::priority_queue<std::pair<double, std::string>, std::vector<std::pair<double, std::string>>, pair_comp> neighbors;
  std::map<std::string, int> labels;

  for (auto desc: references) {
    double d = distance(desc.first, sample, n);
    neighbors.push({d, desc.second});
  }
  
  for (int i=0; i<k; ++i) {
    std::pair<double, std::string> nearest = neighbors.top();
    neighbors.pop();
    labels[nearest.second] += 1;
  }

  int max = 0;
  std::string label;

  for (auto val: labels) {
    if (val.second > max) {
      max = val.second;
      label = val.first;
    } 
  }

  return label;
}
